/*
** load_directory_files.c for pv in /home/saf/pv/src
** 
** Made by Stephan Ferraro
** Login   <saf@xy1.org>
** 
** Started on  Mon Jun 14 19:07:04 2004 Stephan Ferraro
** Last update Tue Sep 14 22:33:54 2004 Stephan Ferraro
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include	<sys/types.h>
#include	<dirent.h>

#include	"pv.h"

int		add_to_char_table(int pos, int *size, t_pv *p, char *msg)
{
  if ((pos + 1) == *size)
    {
      *size += CHAR_TABLE_BUFFER;
      p->av = realloc(p->av, *size * sizeof(char *));
      if (!p->av)
	err_msg_perror("realloc");
    }
  p->av[pos] = msg;
  p->av[pos + 1] = 0;
  return (0);
}

int		my_strcmp(const void *str1, const void *str2)
{
  char		**s1;
  char		**s2;

  s1 = (char **)str1;
  s2 = (char **)str2;
  return (strcmp(*s1, *s2));
}

/* I use quicksort of the glibc for good performance. */
void		sort_table_alphabetically(int number_of_elements, char **table)
{
  qsort(table, number_of_elements, sizeof(char *), my_strcmp); /* we use the fast strcmp to compare the strings */
}

int		read_directory_content(t_pv *p, char *directory, DIR *cur)
{
  struct dirent *dir;
  int		current_element;
  int		ac;
  int		*av_size;
  char		*full_filename;
  int		len;

  current_element = 1; /* backup first space, if image file not found */
  ac = p->ac + 1; /* backup first space, if image file not found */
  av_size = &(p->av_size);
  p->av[0] = ""; /* let free name space in case if find_file(...) has nothing found! */
  len = strlen(directory) + 2; /* + 1 for the "/" + 1 for the 0 at the end */
  while ((dir = readdir(cur)))
    {
      char	*dir_name;

      dir_name = dir->d_name;
      /* Ignore "." and ".." directories. */
      if (!((dir_name[0] == '.' && dir_name[1] == 0) ||
	    (dir_name[0] == '.' && dir_name[1] == '.' && dir_name[2] == 0)))
	{
	  /* Add the filename to the table */
	  full_filename = xmalloc(len + strlen(dir_name));
	  strcpy(full_filename, directory);
	  strcat(full_filename, "/");
	  strcat(full_filename, dir_name);
	  if (!add_to_char_table(current_element, av_size, p, full_filename))
	    current_element++;
	}
    }
  sort_table_alphabetically(current_element - 1, p->av + 1);
  return (current_element);
}

int		find_file(t_pv *p, char *filename)
{
  int		pos;
  char		**tabl;

  pos = 0;
  tabl = p->av;
  DEBUG("Seek for selected filename.");
  while (tabl[pos])
    {
      printf("check file: [%s] :  [%s]\n", filename, tabl[pos]);
      if (!strcmp(filename, tabl[pos]))
	{
	  return (pos - 1);
	}
      pos++;
    }
  return (-2); /* we found no match */
}

/* This function should  be called only once time, or there would be a memory leak! */
/*   (see  p->av = xmalloc(...);, xstrdup etc.) */
void		load_directory_files(t_pv *p)
{
  DIR		*cur;
  char		*current_file;
  char		directory[DIR_SIZE];

  if (!p->ac)
    current_file = 0;
  else
    current_file = p->av[0];
  cur = opendir(".");
  if (!cur)
    err_msg_perror("opendir");
  if (!getcwd(directory, DIR_SIZE))
    err_msg_perror("getcwd");

  p->av_size = CHAR_TABLE_BUFFER;
  p->av = xmalloc(CHAR_TABLE_BUFFER * sizeof(char *));
  p->ac = read_directory_content(p, directory, cur);

  if (current_file)
    {
      int	len;
      char	*full_filename;

      if (!strchr(current_file, '/'))
	{
	  len = strlen(directory) + 2; /* + 1 for the "/" + 1 for the 0 at the end */
	  full_filename = xmalloc(len + strlen(current_file));
	  strcpy(full_filename, directory);
	  strcat(full_filename, "/");
	  strcat(full_filename, current_file);
	  p->current_file_id = find_file(p, full_filename);
	}
      else
	p->current_file_id = find_file(p, current_file);
    }
  else
    p->current_file_id = -1;

  if (p->current_file_id == -2)
    add_to_char_table(0, &(p->av_size), p, current_file);
  if (p->current_file_id != -2 || current_file == 0)
    {
/*       We no more need the first blank entry space. */
      p->ac--;
      p->av++; /* delete blank entry (no more possible to free av after this operation) */
      if (current_file)	/* if there was a filename in parameter */
	p->current_file_id--;
      else
	p->current_file_id = -1;
      p->files_nb--;
    }
  if (p->current_file_id == -2)
    p->current_file_id = -1;
  closedir(cur);
}
