/*
** multiple_files.c for pv in /home/saf/pv
** 
** Made by Stephan Ferraro
** Login   <saf@xy1.org>
** 
** Started on  Mon Dec 29 18:05:05 2003 Stephan Ferraro
** Last update Tue Sep 14 22:25:05 2004 Stephan Ferraro
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include	"pv.h"

void		init_multiple_file(t_pv *p)
{
  p->filename = 0;
  if (p->best_file_id == -1)
    p->best_file_id = p->current_file_id;
}

void		check_and_put_best_filename(t_pv *p, int ret)
{
  if (ret != -1)
    p->best_file_id = p->current_file_id;
  else
    {
      if (p->best_file_id == -1)
	err_msg("\npv: Can't read any files in paramter.");
      else
	{
	  p->current_file_id = p->best_file_id;
	  p->filename =  p->filenames[p->current_file_id];
	}
    }
}

/* This code is a little ugly, but I don't know how I could do it better. :-) */
/* The next file function tries to load next readable valid image file, */
/*   if the image is not readable, it will skip it automatically and switch to next. */
int		next_file(t_pv *p)
{
  int		ret;

  ret = -1;
  init_multiple_file(p);
  if ((p->current_file_id + 1) < p->files_nb)
    {
      p->current_file_id++;
      while (!p->filenames[p->current_file_id] && (p->current_file_id + 1) < p->files_nb)
	p->current_file_id++;
      p->filename = p->filenames[p->current_file_id];
      if (p->filename)
	while ((ret = load_image(p)) != 0)
	  {
	    if (p->verbose)
	      fprintf(stderr, "pv: Can't read file %s [%d] -- ignored\n", p->filename, p->current_file_id);
	    p->filenames[p->current_file_id++] = 0; /* Do no more read this file */
	    if (p->current_file_id == p->files_nb)
	      break;
	    p->filename = p->filenames[p->current_file_id];
	    if (!p->filename)
	      break;
	  }
    }
  check_and_put_best_filename(p, ret);
  return (ret);
}

int		previous_file(t_pv *p)
{
  int		ret;

  ret = -1;
  init_multiple_file(p);
  if (p->current_file_id > 0)
    {
      p->current_file_id--;
      while (!p->filenames[p->current_file_id] && p->current_file_id > 0)
	p->current_file_id--;
      p->filename = p->filenames[p->current_file_id];
      if (p->filename)
	while ((ret = load_image(p)) != 0)
	  {
	    fprintf(stderr, "pv: Can't read file %s [%d] -- ignored\n", p->filename, p->current_file_id);
	    p->filenames[p->current_file_id--] = 0; /* Do no more read this file */
	    if (p->current_file_id <= 0)
	      break;
	    p->filename = p->filenames[p->current_file_id];
	  }
    }
  check_and_put_best_filename(p, ret);
  return (ret);
}

int		aff_next_file(t_pv *p)
{
  if (!next_file(p))
    {
      init_scroll(p);
      aff_current_file(p);
    }
  return (0);
}

int		aff_previous_file(t_pv *p)
{
  if (!previous_file(p))
    {
      init_scroll(p);
      aff_current_file(p);
    }
  return (0);
}

/* We are not sure if the first item is a valid picture, */
/* next_file will check automatically if the next images are valid */
/* images. (auto-ignore if not) */
int		aff_first_file(t_pv *p)
{
  if (p->multiple_files)
    {
      p->current_file_id = -1;
      aff_next_file(p);
    }
  return (0);
}

int		aff_last_file(t_pv *p)
{
  if (p->multiple_files)
    {
      p->best_file_id = -1;
      p->current_file_id = p->files_nb;
      aff_previous_file(p);
    }
  return (0);
}

int		press_previous_file(t_pv *p)
{
  p->next_image = -1;
  return (0);
}

int		press_next_file(t_pv *p)
{
  p->next_image = 1;
  return (0);
}

/* wait_first_img: Wait for 20 cycles in relation with the SDL_Delay in main_loop.c */
/* for next image view (see next_image_if_key_press). */
int		unpress_switch_image(t_pv *p)
{
  p->next_image = 0;
  p->wait_first_cyl = WAIT_FIRST_IMAGE;
  p->wait_first_img = 0;
  return (0);
}

int		next_image_if_key_press(t_pv *p)
{
  if (p->next_image != 0) /* only if a next image key is pressed, proceed. */
    {
      if (p->wait_first_cyl && p->wait_first_img)
	p->wait_first_cyl--;
      else
	{
	  if (p->next_image == -1)
	    aff_previous_file(p);
	  if (p->next_image == 1)
	    aff_next_file(p);
	  p->wait_first_img = 1;
	}
    }
  return (0);
}
