/*
** pv.h for pv in /home/saf/pv
** 
** Made by saf
** Login   <saf@xy1.org>
** 
** Started on  Sat Dec 27 18:06:32 2003 Stephan Ferraro
** Last update Tue Sep 14 22:26:07 2004 Stephan Ferraro
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include			<stdio.h>
#include			<stdlib.h>
#include			<string.h>
#include			<unistd.h>
#include			<X11/Xlib.h>

#include			"SDL.h"
#include			"SDL_image.h"
#include			"SDL_rotozoom.h"
#include			"SDL_gfxPrimitives.h"
#include			"SDL_thread.h"
#include			"SDL_rwops.h"
#include			"SDL_ttf.h"


#include			"macros.h"

/* We need to include here pv_save.h because proto.h contains prototypes */
/* of all C source files. */

#include			"pv_save.h"

#define VERSION_MAJOR		0
#define VERSION_MINOR		5
#define VERSION_MICRO		4

#define	WAIT_FIRST_IMAGE	20	/* 20 cycles to wait (in relation with
					   SDL_Delay in main_loop) to switch
					   to next images. */
#define	WAIT_FIRST_ZOOM		20	/* 20 cycles to wait (in relation with
					   SDL_Delay in main_loop) to switch
					   to next zoom in or out. */

#define	CHAR_TABLE_BUFFER	4096	/* Realocate (char **) table every 4096 entries greater. */
#define	DIR_SIZE		4096	/* A directory name can not be bigger than this value (4096 bytes). */

/* Benchmark structure */
typedef struct		s_benchmark
{
  int			activated;	 /* activated = 1 if we should do a benchmark test */
  struct timeval	start;		 /* start time in miliseconds */
  struct timeval	stop;		 /* stop time in miliseconds */
}			t_benchmark;

/* t_pv PV program main structure */
typedef struct	s_pv
{
  int		ac;		 /* ac arguments number of main() function */
  char		**av;		 /* av arguments of main() function */
  int		av_size;	 /* av entries number */
  t_benchmark	benchmark;	 /* Benchmark test mode with one single image file */
  char		*filename;	 /* Current viewed filename */
  int		fullscreen;	 /* Fullscreen flag (on / off) */
  int		win_w;		 /* X11 Window width in pixels */
  int		win_h;		 /* X11 Window height in pixels */
  int		wm_w;		 /* X-Window-Manager width in pixels */
  int		wm_h;		 /* X-Window-Manager height in pixels */
  int		multiple_files;	 /* Flag, if on, pv was started with more than one file */
  char		**filenames;	 /* If multiple files, it's the table with all files. */
  int		files_nb;	 /* Number of files */
  int		current_file_id; /* Position in the char **filenames of current viewed file */
  int		best_file_id;	 /* First readable image id (index to filenames table) */
  int		auto_resize;	 /* If on, images will fit on screen with auto reszing. */
  int		pos_x;		 /* Current X position of image. */
  int		pos_y;		 /* Current Y position of image. */
  int		done;		 /* If flag on, program exit, all threads are ended up. */
  int		mouse_pressed;	 /* Flag: Mouse button was pressed. */
  int		mouse_x;	 /* X position of mouse cursor. */
  int		mouse_y;	 /* Y position of mouse cursor. */
  int		mouse_old_x;	 /* Last X position of mouse cursor while scrolling. */
  int		mouse_old_y;	 /* Last Y position of mouse cursor while scolling. */
  int		key_press_dir_x; /* Scroll picture by X pixels (pressed with keys) */
  int		key_press_dir_y; /* Scroll picture by Y pixels (pressed with keys) */
  int		help_screen;	 /* Flag: If on, show help screen on screen. */
  int		text_screen;	 /* Flag: If on, show on the left up, file info. */
  int		rotate;		 /* If >= 0, rotate picture. */
  int		zoom;		 /* 100% per default. */
  float         real_zoom;       /* real zoom size */
  int		key_zoom;	 /* scroll key vector (1 = zoom in, -1 = zoom out, 0 = nothing) */
  int		filenames_alloc; /* was malloc used for the filename table? */
  int		next_image;	 /* go to previous or next image */
  int		xfe_mode;	 /* Used for external file managers as like XFE. */
  int		wait_first_img;  /* First next image put on screen without delay. */
  int		wait_first_cyl;  /* Unit: cycles in relation with the delay of main_loop.c */
  int		wait_zoom;	 /* First next zoom in or out on screen without delay. */
  int		wait_zoom_cyl;	 /* Unit: cycles in relation with the delay of main_
loop.c */
  int		zoom_w;		 /* zoom width (virtual image) */
  int		zoom_h;		 /* zoom height (virtual image) */
  int		verbose;	 /* debug mode */
  int		zoom_activated;  /* zoom mode activated: 1 if already zoomed one time (tmp var) */
  int		scroll_now;	 /* scroll flag for next aff_image view, then set to 0 */
  SDL_Rect	scroll_rect;	 /* Is used by scrolling functions. */
  SDL_Surface	*img_file;	 /* Current viewed image file in SDL_Surface format. */
  SDL_Surface	*img_zoom;	 /* img zoom image */
  SDL_Surface	*screen;	 /* Current screen. */
  SDL_Event	*event;
  SDL_RWops	*font_rw;
  TTF_Font	*font;
  SDL_Surface	*font_surface;	/* Current font. */
}		t_pv;

/* Function pointers for key pressing to keep up fast and clean C code. */
typedef struct	s_key
{
  int		key;
  int		(*f)(t_pv *p);
}		t_key;

enum
  {
    PV_DIR_UP,
    PV_DIR_DOWN,
    PV_DIR_RIGHT,
    PV_DIR_LEFT
  };

#define		SCROLL_SIZE	32 /* Pixel jump while scrolling per pressed key */
#define		FONT_SIZE	16 /* TTF font size */

extern char	gl_bin_font_data[171792];

typedef unsigned int	t_uint;
typedef unsigned char	t_uchar;

#include	"proto.h" /* Contains prototypes, this file is generated autmatically */
