/*
** resize.c for pv in /home/saf/pv
** 
** Made by Stephan Ferraro
** Login   <saf@xy1.org>
** 
** Started on  Sun Dec 28 23:47:01 2003 Stephan Ferraro
** Last update Tue Aug 10 17:17:07 2004 
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include	"pv.h"

int		get_zoom_ratio(t_pv *p, int w, int h, int img_w, int img_h)
{
  int		zoom_ratio_h;
  int		zoom_ratio_w;
  int		zoom_final_ratio;

  if (p->rotate == 90 || p->rotate == 270)
    {
      int tmp;

      tmp = img_w;
      img_w = img_h;
      img_h = tmp;
    }
  zoom_ratio_w = (w * 1000) / img_w;
  zoom_ratio_h = (h * 1000) / img_h;

  if (w < p->win_w || h < p->win_h)
    zoom_final_ratio = zoom_ratio_h <= zoom_ratio_w ? zoom_ratio_h : zoom_ratio_w;
  else
    zoom_final_ratio = zoom_ratio_h <= zoom_ratio_w ? zoom_ratio_h : zoom_ratio_w;
  zoom_final_ratio += ((p->zoom * 10) - 1000);
  p->real_zoom = (float)(zoom_final_ratio / 10.0);
  return (zoom_final_ratio);
}

/* Resize the image correctly (do not deform it) and print it to screen. */
void		autoresize(t_pv *p, int w, int h, int img_w, int img_h)
{
  SDL_Rect	rect;
  int		zoom_final_ratio;

  if (p->scroll_now == 0)
    {
      zoom_final_ratio = get_zoom_ratio(p, w, h, img_w, img_h);
      if (p->img_zoom) /* free if already allocated */
	SDL_FreeSurface(p->img_zoom);
      if ((p->img_zoom = rotozoomSurface(p->img_file,
					 (float)p->rotate,
					 (float)((zoom_final_ratio) / 1000.0),
					 SMOOTHING_ON)) == NULL)
	err_msg_function("rotozoomSurface");
      p->zoom_w = p->img_zoom->w;
      p->zoom_h = p->img_zoom->h;
    }
  rect.x = (w - p->img_zoom->w) / 2 + p->scroll_rect.x;
  rect.y = (h - p->img_zoom->h) / 2 + p->scroll_rect.y;
  /* put here later accelerating algorithm, don't need to refresh the complete screen for tiny zoom */
  if ((p->img_zoom->w < w || p->img_zoom->h < h) || p->zoom_activated)
    boxColor(p->screen, 0, 0,
	     w, h, 0x000000ff);
  if (SDL_BlitSurface(p->img_zoom, 0, p->screen, &rect) <= -1)
    err_msg_function("SDL_BlitSurface");
  if (p->text_screen)
    aff_fileinfotext(p);
  if (p->help_screen)
    aff_help_text(p);
  SDL_UpdateRect(p->screen, 0, 0, w, h);

/*   Here the Benchmark test finish */
  if (p->benchmark.activated)
    {
      benchmark_stop(p);
      printf("PV used smoothing technics.\n");
      benchmark_result(p);
      exit(0);
    }
}

void		aff_resized_image(t_pv *p)
{
  autoresize(p, p->event->resize.w, p->event->resize.h, p->img_file->w, p->img_file->h);
  aff_current_file(p);
}

void		resize_window_event(t_pv *p)
{
  SDL_FreeSurface(p->screen);
  p->screen = SDL_SetVideoMode(p->event->resize.w,
			       p->event->resize.h,
			       p->img_file->format->BitsPerPixel,
			       SDL_SWSURFACE | SDL_RESIZABLE);
  p->win_h = p->event->resize.h;
  p->win_w = p->event->resize.w;
  if (p->auto_resize)
    autoresize(p, p->event->resize.w, p->event->resize.h, p->img_file->w, p->img_file->h);
  else
    aff_image(p);
}
