/*
** save.c for pv in /home/saf/pv/src
** 
** Made by Stephan Ferraro
** Login   <saf@xy1.org>
** 
** Started on  Mon Feb  9 00:09:21 2004 Stephan Ferraro
** Last update Thu Feb 12 03:41:25 2004 Stephan Ferraro
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>

#include	"pv.h"

/* Save image as PNG file, usally used when you rotate a picture. */

/* Try 256 different filenames if can't saving to new png file. */
#define		RETRY	256

void		init_save_file(t_save *save, t_pv *p)
{
  char		*new_filename;
  int		i;
  int		retry;
  struct stat	st;

  new_filename = strdup(p->filename);
  for (i = strlen(new_filename); i > 0; i--)
    if (new_filename[i] == '.')
      {
	new_filename[i] = 0;
	break;
      }
  save->filename = xmalloc(strlen(new_filename) + 4096);
  sprintf(save->filename, "%s.png", new_filename);
  if (!lstat(save->filename, &st))
    {
      fprintf(stderr,
	      "File %s.png already exists. Try to save in another filename.\n",
	      new_filename);
      i = 0;
      retry = 1;
      while (retry && i < RETRY)
	{
	  retry = 0;
	  fprintf(stderr, "Try to save to: %s.%d.png\n", new_filename, i);
	  sprintf(save->filename, "%s.%d.png", new_filename, i);
	  if (!lstat(save->filename, &st))
	    retry = 1;
	  i++;
	}
      if (i == RETRY)
	err_msg("Couln't save the new file, possible of wrong file permissions.\n");
    }
  save->fp = fopen(save->filename, "wb");
  if (save->fp == NULL)
    err_msg_perror(save->filename);
  printf("Save new file as: %s\n", save->filename);
}

/* I toke the little documentation in the source code of example.c in */
/* the libpng documentation. */
void		init_png_header(t_save *save, t_pv *p)
{
  png_text	text_ptr[2];

  if ((save->zoom = rotozoomSurface(p->img_file,
			      (float)p->rotate,
			      1.0000,
			      SMOOTHING_OFF)) == NULL)
    err_msg_function("rotozoomSurface");

  /* Create and initialize the png_struct with the desired error handler                   
   * functions.  If you want to use the default stderr and longjump method,                
   * you can supply NULL for the last three parameters.  We also check that                
   * the library version is compatible with the one used at compile time,                  
   * in case we are using dynamically linked libraries.  REQUIRED.                         
   */
  save->png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
					  NULL,
					  NULL,
					  NULL);
  if (!save->png_ptr)
    err_msg_perror("png_create_write_struct");

  /* Allocate/initialize the image information data.  REQUIRED */
  save->info_ptr = png_create_info_struct(save->png_ptr);
  if (!save->info_ptr)
    {
      fclose(save->fp);
      png_destroy_write_struct(&(save->png_ptr),
			       NULL);
      err_msg_perror("png_create_info_struct");
    }
  if (setjmp(png_jmpbuf(save->png_ptr)))
    {
      /* If we get here, we had a problem reading the file */
      fclose(save->fp);
      png_destroy_write_struct(&(save->png_ptr), &(save->info_ptr));
      err_msg_perror("png_jmpbuf");
    }
  png_init_io(save->png_ptr, save->fp);

  /* This is the hard way */
  png_set_IHDR(save->png_ptr,
	       save->info_ptr,
	       save->zoom->w,
	       save->zoom->h,
	       8, /* bits per color channel */
	       PNG_COLOR_TYPE_RGB,
	       PNG_INTERLACE_NONE,
	       PNG_COMPRESSION_TYPE_BASE,
	       PNG_FILTER_TYPE_BASE);

  /* Optional gamma chunk is strongly suggested if you have any guess                      
   * as to the correct gamma of the image.                                                 
   */
/*   png_set_gAMA(png_ptr, info_ptr, gamma); */

  /* Optionally write comments into the image */
  text_ptr[0].key = "Encoder";
  text_ptr[0].text = "Image file saved with pv!";
  text_ptr[0].compression = PNG_TEXT_COMPRESSION_NONE;
  text_ptr[1].key = "URL";
  text_ptr[1].text = "http://trashmail.net/pv/";
  text_ptr[1].compression = PNG_TEXT_COMPRESSION_NONE;
#ifdef PNG_iTXt_SUPPORTED
  text_ptr[0].lang = NULL;
  text_ptr[1].lang = NULL;
  text_ptr[2].lang = NULL;
#endif
  png_set_text(save->png_ptr, save->info_ptr, text_ptr, 2);

  /* other optional chunks like cHRM, bKGD, tRNS, tIME, oFFs, pHYs, */
  /* note that if sRGB is present the gAMA and cHRM chunks must be ignored                 
   * on read and must be written in accordance with the sRGB profile */

  /* Write the file header information.  REQUIRED */
  png_write_info(save->png_ptr, save->info_ptr);
}

t_uchar		*convert_32bits_to_24bits(t_uchar *pixels, t_uint w, t_uint h)
{
  t_uint	i;
  t_uint	u;
  t_uint	len;
  t_uchar	*new;

  len = w * h * 4; /* input is a 32 bits image (4 bytes per pixel) */
  new = xmalloc(w * h * 3); /* new image in 24 bits (3 bytes per pixel) */
  for (i = 0, u = 0; i < len; i++)
    {
      if (i % 4 != 3)
	new[u++] = pixels[i];
    }
  return (new);
}

/*   Save picture data */
void		init_png_save_core(t_save *save, t_pv *p)
{
  png_uint_32	k;
  png_uint_32	height;
  png_uint_32	width;
  png_byte	*image;
  png_bytep	*row_pointers;
  void		*tmp;
  int		bytesperpixel;

  width = save->zoom->w;
  height = save->zoom->h;
  bytesperpixel = 3; /* 24 bits */
  image = convert_32bits_to_24bits(save->zoom->pixels, save->zoom->w, save->zoom->h);
  tmp = xmalloc(save->zoom->h * sizeof(png_bytep));
  row_pointers = (png_byte **)tmp;

  for (k = 0; k < height; k++)
    row_pointers[k] = (image + (k * width * bytesperpixel));
  png_write_image(save->png_ptr, row_pointers);

  png_write_end(save->png_ptr, save->info_ptr);
  png_destroy_write_struct(&save->png_ptr, &save->info_ptr);
  free(tmp);
  free(image);
  SDL_FreeSurface(save->zoom);
  free(save->png_ptr);
}

int		press_save_key(t_pv *p)
{
  t_save	save;

  memset(&save, 0, sizeof(save));
  init_save_file(&save, p);
  init_png_header(&save, p);
  init_png_save_core(&save, p);
  fclose(save.fp);
  add_to_filelist(p, save.filename);
  SDL_UpdateRect(p->screen, 0, 0, p->screen->w, p->screen->h);
  return (0);
}
